@@ -3,15 +3,19 @@ use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::iter::{Product, Sum};
 use std::ops::*;
+use std::str::FromStr;
 
+use async_trait::async_trait;
 use collate::Collate;
-use destream::{EncodeSeq, Encoder, IntoStream, ToStream};
+use destream::{Decoder, EncodeSeq, Encoder, FromStream, IntoStream, ToStream};
+use futures::TryFutureExt;
 use num::traits::Pow;
 use safecast::*;
 use serde::ser::{Serialize, SerializeSeq, Serializer};
 
 use super::class::*;
-use super::{Number, _Complex};
+use super::{Error, Number, _Complex};
+use serde::{Deserialize, Deserializer};
 
 /// A boolean value.
 #[derive(Clone, Copy, Hash, Ord, PartialEq, PartialOrd)]
@@ -71,6 +75,14 @@ impl From<bool> for Boolean {
     }
 }
 
+impl FromStr for Boolean {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        bool::from_str(s).map(Self::from).map_err(Error::new)
+    }
+}
+
 impl From<Boolean> for bool {
     fn from(b: Boolean) -> bool {
         b.0
@@ -199,6 +211,27 @@ impl Serialize for Boolean {
     }
 }
 
+impl<'de> Deserialize<'de> for Boolean {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        bool::deserialize(deserializer).map(Self::from)
+    }
+}
+
+#[async_trait]
+impl FromStream for Boolean {
+    type Context = ();
+
+    async fn from_stream<D: Decoder>(
+        cxt: Self::Context,
+        decoder: &mut D,
+    ) -> Result<Self, D::Error> {
+        bool::from_stream(cxt, decoder).map_ok(Self::from).await
+    }
+}
+
 impl<'en> ToStream<'en> for Boolean {
     fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
         e.encode_bool(self.0)
@@ -262,6 +295,22 @@ impl NumberInstance for Complex {
     }
 }
 
+impl FloatInstance for Complex {
+    fn is_infinite(&self) -> bool {
+        match self {
+            Self::C32(c) => c.im.is_infinite() || c.re.is_infinite(),
+            Self::C64(c) => c.im.is_infinite() || c.re.is_infinite(),
+        }
+    }
+
+    fn is_nan(&self) -> bool {
+        match self {
+            Self::C32(c) => c.im.is_nan() || c.re.is_nan(),
+            Self::C64(c) => c.im.is_nan() || c.re.is_nan(),
+        }
+    }
+}
+
 impl CastFrom<[f32; 2]> for Complex {
     fn cast_from(arr: [f32; 2]) -> Self {
         Self::C32(num::Complex::new(arr[0], arr[1]))
@@ -533,6 +582,26 @@ impl From<_Complex<f64>> for Complex {
     }
 }
 
+impl FromStr for Complex {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        num::Complex::<f64>::from_str(s)
+            .map(Self::from)
+            .map_err(Error::new)
+    }
+}
+
+impl<'de> Deserialize<'de> for Complex {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let [re, im]: [f64; 2] = Deserialize::deserialize(deserializer)?;
+        Ok(num::Complex::new(re, im).into())
+    }
+}
+
 impl Serialize for Complex {
     fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
         match self {
@@ -552,6 +621,19 @@ impl Serialize for Complex {
     }
 }
 
+#[async_trait]
+impl FromStream for Complex {
+    type Context = ();
+
+    async fn from_stream<D: Decoder>(
+        cxt: Self::Context,
+        decoder: &mut D,
+    ) -> Result<Self, D::Error> {
+        let [re, im]: [f64; 2] = FromStream::from_stream(cxt, decoder).await?;
+        Ok(num::Complex::new(re, im).into())
+    }
+}
+
 impl<'en> ToStream<'en> for Complex {
     fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
         match self {
@@ -670,6 +752,22 @@ impl NumberInstance for Float {
     }
 }
 
+impl FloatInstance for Float {
+    fn is_infinite(&self) -> bool {
+        match self {
+            Self::F32(f) => f.is_infinite(),
+            Self::F64(f) => f.is_infinite(),
+        }
+    }
+
+    fn is_nan(&self) -> bool {
+        match self {
+            Self::F32(f) => f.is_nan(),
+            Self::F64(f) => f.is_nan(),
+        }
+    }
+}
+
 impl Eq for Float {}
 
 impl Add for Float {
@@ -859,6 +957,14 @@ impl From<UInt> for Float {
     }
 }
 
+impl FromStr for Float {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        f64::from_str(s).map(Self::from).map_err(Error::new)
+    }
+}
+
 impl CastFrom<Complex> for Float {
     fn cast_from(c: Complex) -> Float {
         use Complex::*;
@@ -900,6 +1006,15 @@ impl From<Float> for f64 {
     }
 }
 
+impl<'de> Deserialize<'de> for Float {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        f64::deserialize(deserializer).map(Self::from)
+    }
+}
+
 impl Serialize for Float {
     fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
         match self {
@@ -909,6 +1024,18 @@ impl Serialize for Float {
     }
 }
 
+#[async_trait]
+impl FromStream for Float {
+    type Context = ();
+
+    async fn from_stream<D: Decoder>(
+        cxt: Self::Context,
+        decoder: &mut D,
+    ) -> Result<Self, D::Error> {
+        f64::from_stream(cxt, decoder).map_ok(Self::from).await
+    }
+}
+
 impl<'en> ToStream<'en> for Float {
     fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
         match *self {
@@ -963,7 +1090,7 @@ impl Collate for FloatCollator {
             } else if left == f32::INFINITY || right == f32::NEG_INFINITY {
                 Ordering::Greater
             } else {
-                panic!("No collation defined between {} and {}", left, right)
+                panic!("no collation defined between {} and {}", left, right)
             }
         }
     }
@@ -1333,6 +1460,23 @@ impl From<Int> for i64 {
     }
 }
 
+impl FromStr for Int {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        i64::from_str(s).map(Self::from).map_err(Error::new)
+    }
+}
+
+impl<'de> Deserialize<'de> for Int {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        i64::deserialize(deserializer).map(Self::from)
+    }
+}
+
 impl Serialize for Int {
     fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
         match self {
@@ -1344,6 +1488,18 @@ impl Serialize for Int {
     }
 }
 
+#[async_trait]
+impl FromStream for Int {
+    type Context = ();
+
+    async fn from_stream<D: Decoder>(
+        cxt: Self::Context,
+        decoder: &mut D,
+    ) -> Result<Self, D::Error> {
+        i64::from_stream(cxt, decoder).map_ok(Self::from).await
+    }
+}
+
 impl<'en> ToStream<'en> for Int {
     fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
         match *self {
@@ -1777,6 +1933,23 @@ impl From<UInt> for usize {
     }
 }
 
+impl FromStr for UInt {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        u64::from_str(s).map(Self::from).map_err(Error::new)
+    }
+}
+
+impl<'de> Deserialize<'de> for UInt {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        u64::deserialize(deserializer).map(Self::from)
+    }
+}
+
 impl Serialize for UInt {
     fn serialize<S: Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
         match self {
@@ -1788,6 +1961,18 @@ impl Serialize for UInt {
     }
 }
 
+#[async_trait]
+impl FromStream for UInt {
+    type Context = ();
+
+    async fn from_stream<D: Decoder>(
+        cxt: Self::Context,
+        decoder: &mut D,
+    ) -> Result<Self, D::Error> {
+        u64::from_stream(cxt, decoder).map_ok(Self::from).await
+    }
+}
+
 impl<'en> ToStream<'en> for UInt {
     fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
         match *self {
